// src/create-external-subject.ts
function defaultShouldUpdate(a, b) {
  return !Object.is(a, b);
}
function createExternalSubject(options) {
  var _a;
  const shouldUpdate = (_a = options.shouldUpdate) != null ? _a : defaultShouldUpdate;
  let cache = options.read();
  const listeners = new Set();
  let request;
  const requestUpdate = () => {
    if (shouldUpdate(cache, options.read())) {
      if (request) {
        request.alive = false;
      }
      request = {
        promise: Promise.resolve().then(() => {
          if (request == null ? void 0 : request.alive) {
            request = void 0;
            cache = options.read();
            listeners.forEach((listener) => {
              listener();
            });
          }
        }),
        alive: true
      };
    }
  };
  let unsubscribe;
  const subscribe = (handler) => {
    listeners.add(handler);
    if (options.lazySubscribe && options.subscribe && listeners.size === 1) {
      const unsub = options.subscribe(requestUpdate);
      if (unsub) {
        unsubscribe = unsub;
      }
    }
    return () => {
      listeners.delete(handler);
      if (options.lazySubscribe && unsubscribe) {
        unsubscribe();
      }
    };
  };
  if (!options.lazySubscribe && options.subscribe) {
    unsubscribe = options.subscribe(requestUpdate);
  }
  const destroy = () => {
    if (request) {
      request.alive = false;
    }
    listeners.clear();
    if (unsubscribe) {
      unsubscribe();
    }
  };
  return {
    subscribe,
    shouldUpdate,
    requestUpdate,
    getRequest: () => request,
    getCachedValue: () => cache,
    getCurrentValue: options.read,
    destroy
  };
}

// src/use-external-subject.ts
import {useForceUpdate, useIsomorphicEffect} from "@lyonph/react-hooks";
import {useDebugValue} from "react";
function useExternalSubjectInternal(subject, suspense = false) {
  const forceUpdate = useForceUpdate();
  useIsomorphicEffect(() => {
    const unsubscribe = subject.subscribe(forceUpdate);
    return unsubscribe;
  }, [subject]);
  useIsomorphicEffect(() => {
    if (!subject.getRequest()) {
      subject.requestUpdate();
      const ongoing2 = subject.getRequest();
      if (ongoing2) {
        forceUpdate();
        return void 0;
      }
    }
    const timeout = setTimeout(() => {
      const afterOngoing = subject.getRequest();
      if (afterOngoing) {
        forceUpdate();
        return;
      }
      subject.requestUpdate();
      const currentOngoing = subject.getRequest();
      if (currentOngoing) {
        forceUpdate();
      }
    });
    return () => {
      clearTimeout(timeout);
    };
  });
  const cached = subject.getCachedValue();
  const ongoing = subject.getRequest();
  if (ongoing) {
    if (suspense) {
      throw ongoing.promise;
    }
    return cached;
  }
  subject.requestUpdate();
  const current = subject.getRequest();
  if (current) {
    if (suspense) {
      throw current.promise;
    }
    return cached;
  }
  return subject.getCurrentValue();
}
function useExternalSubject(subject, suspense = false) {
  const value = useExternalSubjectInternal(subject, suspense);
  useDebugValue(value);
  return value;
}
export {
  createExternalSubject,
  useExternalSubject
};
//# sourceMappingURL=react-external-subject.esm.js.map
